function autoBreak(babel, options = {}) {
    const { types: t, transform } = babel;
    const { timeout = 2000, onBreak = noop } = options;
    const onAutoBreak = transformOnBreak({ t, transform, onBreak });
    return {
        visitor: {
            ForStatement: breakAfter,
            WhileStatement: breakAfter,
            DoWhileStatement: breakAfter
        }
    };
    function breakAfter(loop) {
        var _a;
        const { line, column } = Object.assign({ line: 0, column: 0 }, (_a = loop.node.loc) === null || _a === void 0 ? void 0 : _a.start);
        const id = loop.scope.generateUidIdentifier('auto_break_start');
        loop.insertBefore(generateInitialization({ t, id }));
        const loopBody = loop.get('body');
        const loopGuard = generateLoopGuard({ t, id, line, column, timeout, onAutoBreak });
        if (!loopBody.isBlockStatement()) {
            loopBody.replaceWith(t.blockStatement([loopGuard, loopBody.node]));
        }
        else {
            loopBody.unshiftContainer('body', loopGuard);
        }
    }
}
function transformOnBreak({ t, transform, onBreak }) {
    var _a, _b;
    const onAutoBreakCode = typeof onBreak === 'function'
        ? onBreak.toString().replace(/^function\s*\(/, 'function $onAutoBreak$(')
        : `() => console.error("${String(onBreak).replace(/"/g, '\\"')}")`;
    const onAutoBreak = (_b = (_a = transform(onAutoBreakCode, { ast: true })) === null || _a === void 0 ? void 0 : _a.ast) === null || _b === void 0 ? void 0 : _b.program.body[0];
    if (t.isExpressionStatement(onAutoBreak)) {
        // ArrowFunctionExpression
        return onAutoBreak.expression;
    }
    if (t.isFunctionDeclaration(onAutoBreak)) {
        return t.functionExpression(onAutoBreak.id, onAutoBreak.params, onAutoBreak.body);
    }
    /* c8 ignore next */
    throw new Error('babel-babel-auto-break: invalid onBreak action'); // unreachable
}
const noop = function () { };
function generateInitialization({ t, id }) {
    return (t.variableDeclaration('var', [
        t.variableDeclarator(id, t.callExpression(t.memberExpression(t.identifier('Date'), t.identifier('now')), []))
    ]));
}
function generateLoopGuard({ t, id, line, column, timeout, onAutoBreak }) {
    return (t.ifStatement(t.binaryExpression('>', t.binaryExpression('-', t.callExpression(t.memberExpression(t.identifier('Date'), t.identifier('now')), []), id), t.numericLiteral(timeout)), t.blockStatement([
        t.expressionStatement(t.callExpression(onAutoBreak, [
            t.numericLiteral(line),
            t.numericLiteral(column)
        ])),
        t.breakStatement()
    ])));
}

export { autoBreak as default };
